use strict;
use XML::SAX::ParserFactory;

$XML::SAX::ParserPackage = "XML::SAX::PurePerl";

my $handler = MyHandler->new();
my $parser = XML::SAX::ParserFactory->parser(Handler => $handler);
my $inputXmlFile = shift || "ch3_pureperl.xml";
my %parser_args = (Source => {SystemId => $inputXmlFile});
$parser->parse(%parser_args);

package MyHandler;
my ($current_element, $total, %projectHours, $currentProj);

sub new {
  my $type = shift;
  return bless {}, $type;
}

# start_document event handler
sub start_document {
  my $key;

  print "\nTimecard Report\n";
  print "---------------\n";
}

sub start_element {
  my ($self, $element) = @_;

  # Set $current_element to the current element.
  $current_element = $element->{Name};

  my %atts = %{$element->{Attributes}};
  my $numAtts = keys(%atts);

  # Check to see if this element has attributes.
  if ($numAtts > 0) {
    print "\n";

    my ($thisAtt, $val);
    for my $key (keys %atts) {

      # Use this to look for a particular attribute.
      if ($atts{$key}->{Name} eq 'name') {
	print "Name: $atts{$key}->{Value}\n";
      }
      elsif ($atts{$key}->{Name} eq 'employee_num') {
	print "Employee Number: $atts{$key}->{Value}\n";
      }
    }
  }
}

# characters event handler
sub characters {
  my ($self, $characters) = @_;

  my $char_data = $characters->{Data};

  # Remove leading and trailing whitespace.
  $char_data =~ s/^\s*//;
  $char_data =~ s/\s*$//;

  if (length($char_data) ) {

    # Look for a particular element.
    if (($current_element eq 'project_number') ) {
      print "Project Number: $char_data\t";
      $currentProj = $char_data;
    }
    elsif ($current_element eq 'hours_charged') {
      if (exists ($projectHours{$currentProj}) ) {
   	$projectHours{$currentProj} += $char_data;
      }
      else {
   	$projectHours{$currentProj} = $char_data;
      }

      print "Hours: $char_data\n";

      # Increment the total hours here.
      $total += $char_data;
    }
  }
}

# end_element event handler
sub end_element {
  my ($self, $element) = @_;

  if ($element) {

    # We're at the end of an employee element, so
    # print out the total hours for this employee
    # and reset the scalar $total to 0.
    if (($element->{Name}) eq 'employee') {
      print "Total Hours: $total\n";
      $total = 0;
    }
  }
}

# end_document event handler
sub end_document {
  my $key;

  # Print out any summary information here.
  print "\nProject Summary\n";
  print "---------------\n";
  foreach $key (sort keys %projectHours) {
    print "Project $key charged a total of $projectHours{$key} hours\n"
  }
}
