use strict;
use XML::Xerces;

my $file = "ch3_atm_log.xml";
my ($deposit_count, $withdrawal_count);

package EventHandler;
use strict;
use vars qw(@ISA);
@ISA = qw(XML::Xerces::PerlContentHandler);

my ($trans_type, $total_deps, $total_withdrawals, $current_element);

sub start_document {
    print "ATM Summary Report\n";
}

sub start_element {
  my ($self,$uri,$localname,$qname,$attrs) = @_;
  my $attVal;

  $current_element = $localname;

  if ($attrs->getLength > 0) {

      if ($localname eq "atm_log") {	  
	  $attVal = $attrs->getValue("date");
	  print "Date: $attVal\n\n";
	  print "--------------------\n";
      }

      if ($localname eq "transaction") {
	  $trans_type = $attrs->getValue("type");
      }
  }
}

sub characters {
  my ($self,$str,$len) = @_;
  $self->{chars} += $len;

  if ($current_element eq "name") {
      print "Name: $str\n";
  }

  if ($current_element eq "account") {
      print "Account Number: $str\n";
  }

  if ($current_element eq "amount") {
      
      if ($trans_type eq "deposit") {
	  $total_deps += $str;
	  ++$deposit_count;
      }      
      elsif ($trans_type eq "withdrawal") {
	  $total_withdrawals += $str;
	  ++$withdrawal_count;
      }
      print "Amount: \$$str\n";
  }
}

sub end_element {
  my ($self,$uri,$localname,$qname) = @_;

  if ($localname eq "transaction") {
      print "--------------------\n";
  }
}

sub end_document {
    my $total_transactions = $deposit_count + $withdrawal_count;
    my $output_dep = sprintf("%.2f", $total_deps);
    my $output_wd = sprintf("%.2f", $total_withdrawals);

    print "Total of $total_transactions processed today\n\n";

    print "Transaction Summary\n";
    print "Received: $deposit_count deposits\n";
    print "Total Value of Deposits: \$$output_dep\n\n";
    
    print "Received: $withdrawal_count withdrawals\n";
    print "Total Value of Withdrawals: \$$output_wd\n";
}

package main;
my $parser = XML::Xerces::XMLReaderFactory::createXMLReader();

eval {
    $parser->setFeature("http://xml.org/sax/features/validation", 1);
};

if ($@) {
    if (ref $@) {
	die $@->getMessage();
    } else {
	die $@;
    }
}

my $error_handler = XML::Xerces::PerlErrorHandler->new();
$parser->setErrorHandler($error_handler);

my $event_handler = EventHandler->new();
$parser->setContentHandler($event_handler);

eval {
  $parser->parse (XML::Xerces::LocalFileInputSource->new($file));
};

if ($@) {
    if (ref $@) {
	die $@->getMessage();
    } else {
	die $@;
    }
}

exit;





